"use strict";
this.name	= "ils";
this.author	= "Norby";
this.copyright	= "2014 Norby";
this.description= "Instrument Landing System";
this.version	= "1.3";
this.licence	= "CC BY-NC-SA 4.0";


//internal properties, should not touch
this.$ILSF = null; //FCB pointer
this.$ILSP = null; //previous heading in FCB
this.$ILST = null; //Towbar worldScript pointer
this.$ILSS = null; //the target station

//worldscript events
this.startUp = function() {
	this.$ILST = worldScripts["towbar"];
	if( !isValidFrameCallback( this.$ILSF ) )
		this.$ILSF = addFrameCallback( this.$FCB.bind(this) );
}

this.equipmentDamaged = this.equipmentDestroyed = function(equipment) {
	if(equipment == "EQ_ILS") this.$ClearILS(); //stop ILS if working
}

this.shipTargetAcquired = function() {
}

/* another oxp cause a bug which turn off ILS immediately in the next lines so do not use this event
this.shipTargetLost = function() {
	log(this.name, "ILS OFF 1");//debug
	this.$ClearILS();
}
*/

this.shipWillDockWithStation = this.shipWillEnterWitchspace = this.shipDied = function() {
	this.$ClearILS(true);//nomsg
}

//ILS methods
this.$ClearILS = function(nomsg) {
	this.$ILSS = null;
	this.$ILSP = null;
	if( this.$ILSV && this.$ILSV.isVisualEffect ) {
		this.$ILSV.remove();
		if(!nomsg) player.consoleMessage("ILS off");
	}
	this.$ILSV = null;
//	if( isValidFrameCallback( this.$ILSF ) ) removeFrameCallback( this.$ILSF );
//	log(this.name, "ILS OFF");//debug
}

this.$FCB = function( delta ) {
//	var w = worldScripts.ils;
//	player.consoleMessage(this.$ILSS+" "+delta); //debug
	var p = player.ship;
	var t = p.target;
	if( p && t && t.isValid && t.isStation  //target is station,
		&& !p.missilesOnline //not in missile targeting mode
		&& p.equipmentStatus("EQ_ILS") == "EQUIPMENT_OK" //has ILS
		&& t.dataKey.indexOf("carrier") == -1 ) { //target is not a carrier

		if( this.$ILSS != t ) { //first lock on this target
			this.$ILSS = t; //target station
			player.consoleMessage("ILS set to "+t.displayName, 5);
//			log(this.name, "ILS set to: "+t);//debug
		}
		
		var ph = p.heading;
		if( !this.$ILSP ) this.$ILSP = ph;
		var pp = p.position;
		var s = this.$ILSS; //station
		var mp = p.maxPitch;
		var mr = p.maxRoll;
		var my = p.maxYaw;
//		log(this.name, "Station: "+s);//debug

		var u = s.subEntities;
		var i = 0;
		if( u ) while( u[i] && !u[i].isDock ) i++;
		var bp = s.position;
		if( u[i] && u[i].isDock ) {
//			var dd = u[i].position.magnitude(); //dock dist. from the center of the station
//			bp = s.position.add( s.heading.direction().multiply(dd) ); //old formula
			bp = s.position.add( //better formula for non-centered docks
				Vector3D( s.heading.direction().multiply( u[i].position.z ),
				s.vectorUp.direction().multiply( u[i].position.y ),
				s.vectorRight.direction().multiply( u[i].position.x ) ) );
		}
		var d = pp.distanceTo( bp ) / 2;

		var w = s.position.subtract( pp ).direction();
		var an = s.heading.angleTo(w);
//		player.consoleMessage(d +" "+ s.heading.angleTo(w)); //debug
		if( d < 100 || ( an < 1.57 && d < 300 ) ) //fix for rock hermit
			d = -500; //for sure, do not turn back if crossed the point in the dock
		else {
			if( an < 3.14 ) { //player is not before the station yet
				var na = 3.14-an;
				na = na * na;
				d += na * 2000;
				//extend distance if apporaching behind from the station
			}
			d = Math.min( 9500, d ); //max. at the Nav Buoy
		}
		var a = bp.add( s.heading.direction().multiply( d ) ); //aimed position

		var v = a.subtract( pp ).direction();
		if( this.$ILSV && this.$ILSV.isVisualEffect )
			this.$ILSV.position = a;
		else this.$ILSV = system.addVisualEffect("ils-marker", a);
		
		var angle = ph.angleTo(v);

		var ato = 0;
		if(ph && this.$ILSP) ato = ph.angleTo(this.$ILSP);//angle to the Previous Heading
		if( ato < 0.005 && angle > 0.0015
			 //steer if no manual steering and not in 0.1 degree
			&& p.speed > 1 ) { //and player is not stopped
			//if the above ato value lower then can not start steering in my intel atom netbook
//			player.consoleMessage(this.$ILSP +" vs "+ ph); //debug
			var r = Math.min(p.maxPitch*delta, angle/12);//half max turn/step and not too accutate

			if( this.$ILST && this.$ILST.$TowbarShip
				&& this.$ILST.$TowbarShip.isValid ) {
				var ma = Math.min( 2, p.mass / this.$ILST.$TowbarShip.mass ); ///small ship max. 2x
				r = r * ma / 3; // 1/3 of the original rate with same mass, min. 1/5 max. 2/3
//				player.consoleMessage("ILS slow steering with towed ship "+ato);//debug
			}
			
			var c = ph.cross(v).direction(); //set the plane where we should rotate in
			var q = p.orientation.rotate( c, -r );
			p.orientation = q;
		}
		
		this.$ILSP = p.heading;
	} else this.$ClearILS(); //no target or a new target is not station, etc.
}